@@ -1,7 +1,7 @@
 package App::wordlist;
 
 our $DATE = '2016-04-15'; # DATE
-our $VERSION = '0.23'; # VERSION
+our $VERSION = '0.24'; # VERSION
 
 use 5.010001;
 use strict;
@@ -72,7 +72,6 @@ $SPEC{wordlist} = {
         },
         wordlists => {
             'x.name.is_plural' => 1,
-            'x.name.singular' => 'wordlist',
             schema => ['array*' => of => 'str*'],
             summary => 'Select one or more wordlist modules',
             cmdline_aliases => {w=>{}},
@@ -88,7 +87,7 @@ $SPEC{wordlist} = {
             },
         },
         or => {
-            summary => 'Use OR logic instead of the default AND',
+            summary => 'Match any word in query instead of the default "all"',
             schema  => 'bool',
         },
         action => {
@@ -115,11 +114,19 @@ $SPEC{wordlist} = {
             summary => 'Use local CPAN mirror first when available (for -L)',
         },
         detail => {
-            summary => 'Display more information when listing modules',
+            summary => 'Display more information when listing modules/result',
+            description => <<'_',
+
+When listing installed modules (`-l`), this means also returning a wordlist's
+type and language.
+
+When returning grep result, this means also returning wordlist name.
+
+_
             schema  => 'bool',
         },
-        type => {
-            summary => 'Only include wordlists of certain type',
+        types => {
+            summary => 'Only include wordlists of certain type(s)',
             description => <<'_',
 
 By convention, type information is encoded in the wordlist's name. `Char` means
@@ -129,7 +136,7 @@ wordlists that are not of type `Char` and `Phrase`. `Test` means to only include
 wordlists with names matching `Test::*`.
 
 _
-            schema => ['str*', in=>['Char', 'Phrase', 'Word', 'Test']],
+            schema => ['array*', of => ['str*', in=>['Char', 'Phrase', 'Word', 'Test']]],
             description => <<'_',
 
 By convention, language information is encoded in the wordlist's name. For
@@ -139,10 +146,11 @@ example, English wordlists have names matching `EN::*` or `Word::EN::*` or
 _
             cmdline_aliases => {t=>{}},
         },
-        lang => {
-            summary => 'Only include wordlists of certain language',
-            schema => ['str*', match => '\A\w\w\z'],
-            completion => sub {
+        langs => {
+            'x.name.is_plural' => 1,
+            summary => 'Only include wordlists of certain language(s)',
+            schema => ['array*', of => ['str*', match => '\A\w\w\z']],
+            element_completion => sub {
                 my %args = @_;
                 my @langs;
                 for my $rec (@{ _list_installed() }) {
@@ -183,13 +191,13 @@ _
         },
         {
             argv => [qw/-t Phrase foo/],
-            summary => 'Select phrase wordlists',
+            summary => 'Select phrase wordlists (multiple -t allowed)',
             test => 0,
             'x.doc.show_result' => 0,
         },
         {
             argv => [qw/--lang FR foo/],
-            summary => 'Select French wordlists',
+            summary => 'Select French wordlists (multiple --lang allowed)',
             test => 0,
             'x.doc.show_result' => 0,
         },
@@ -224,6 +232,7 @@ sub wordlist {
     my $ci = $args{ignore_case} // 1;
     my $or = $args{or};
     my $arg = $args{arg} // [];
+    my $detail = $args{detail};
 
     if ($action eq 'grep') {
         # convert /.../ in arg to regex
@@ -243,11 +252,11 @@ sub wordlist {
         } else {
             $wordlists = [];
             for my $rec (@$list_installed) {
-                if ($args{type}) {
-                    next unless $rec->{type} eq $args{type};
+                if ($args{types} && @{ $args{types} }) {
+                    next unless grep { $rec->{type} eq $_ } @{$args{types}};
                 }
-                if ($args{lang}) {
-                    next unless $rec->{lang} eq uc($args{lang});
+                if ($args{langs} && @{ $args{langs} }) {
+                    next unless grep { $rec->{lang} eq uc($_) } @{$args{langs}};
                 }
                 push @$wordlists, $rec->{name};
             }
@@ -276,7 +285,7 @@ sub wordlist {
                         if ($or) {
                             # succeed early when --or
                             if ($match) {
-                                push @res, $word;
+                                push @res, $detail ? [$wl, $word] : $word;
                                 return;
                             }
                         } else {
@@ -287,7 +296,7 @@ sub wordlist {
                         }
                     }
                     if (!$or || !@$arg) {
-                        push @res, $word;
+                        push @res, $detail ? [$wl, $word] : $word;
                     }
                 }
             );
@@ -298,14 +307,14 @@ sub wordlist {
 
         my @res;
         for (@$list_installed) {
-            if ($args{detail}) {
+            if ($detail) {
                 push @res, $_;
             } else {
                 push @res, $_->{name};
             }
         }
         [200, "OK", \@res,
-         {('cmdline.default_format' => 'text') x !!$args{detail}}];
+         {('cmdline.default_format' => 'text') x !!$detail}];
 
     } elsif ($action eq 'list_cpan') {
 
@@ -372,7 +381,7 @@ App::wordlist - Grep words from WordList::*
 
 =head1 VERSION
 
-This document describes version 0.23 of App::wordlist (from Perl distribution App-wordlist), released on 2016-04-15.
+This document describes version 0.24 of App::wordlist (from Perl distribution App-wordlist), released on 2016-04-15.
 
 =head1 SYNOPSIS
 
@@ -405,13 +414,13 @@ Examples:
 
  wordlist( arg => ["foo"], wordlists => ["ID::KBBI"]);
 
-=item * Select phrase wordlists:
+=item * Select phrase wordlists (multiple -t allowed):
 
- wordlist( arg => ["foo"], type => "Phrase");
+ wordlist( arg => ["foo"], types => ["Phrase"]);
 
-=item * Select French wordlists:
+=item * Select French wordlists (multiple --lang allowed):
 
- wordlist( arg => ["foo"], lang => "FR");
+ wordlist( arg => ["foo"], langs => ["FR"]);
 
 =item * Filter by regex:
 
@@ -439,13 +448,18 @@ Arguments ('*' denotes required arguments):
 
 =item * B<detail> => I<bool>
 
-Display more information when listing modules.
+Display more information when listing modules/result.
+
+When listing installed modules (C<-l>), this means also returning a wordlist's
+type and language.
+
+When returning grep result, this means also returning wordlist name.
 
 =item * B<ignore_case> => I<bool> (default: 1)
 
-=item * B<lang> => I<str>
+=item * B<langs> => I<array[str]>
 
-Only include wordlists of certain language.
+Only include wordlists of certain language(s).
 
 =item * B<lcpan> => I<bool>
 
@@ -459,11 +473,11 @@ Use local CPAN mirror first when available (for -L).
 
 =item * B<or> => I<bool>
 
-Use OR logic instead of the default AND.
+Match any word in query instead of the default "all".
 
-=item * B<type> => I<str>
+=item * B<types> => I<array[str]>
 
-Only include wordlists of certain type.
+Only include wordlists of certain type(s).
 
 By convention, language information is encoded in the wordlist's name. For
 example, English wordlists have names matching C<EN::*> or C<Word::EN::*> or
